'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _dispatcher = require('./dispatcher');

var _dispatcher2 = _interopRequireDefault(_dispatcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Primus = window.Primus;

exports.default = function () {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  return function (store) {
    // initialise network io
    console.log('[io] connecting...');

    var ioOptions = {},
        io = Primus.connect('http://localhost:3000', ioOptions),
        gossip = new _dispatcher2.default(),
        gossipStream = gossip.createStream(),
        history = { actions: [], state: [] };

    // the internet is a series of tubes

    // spark.pipe(docStream).pipe(spark)

    io.on('data', function message(data) {
      // console.log('[io] <-', data)
      gossipStream.write(data);
    });

    gossipStream.on('data', function (data) {
      // console.log('[io] ->', data)
      io.write(data);
    });

    // network events

    io.on('open', function () {
      console.log('[io] connection open');
    });

    io.on('error', function (error) {
      console.log('[io] error', error);
    });

    // store stream events

    gossipStream.on('error', function (error) {
      console.warn('[gossip] error', error);
    });

    // handshake header recieved from a new peer. includes their id and clock info
    gossipStream.on('header', function (header) {
      var id = header.id;
      var clock = header.clock;

      console.log('[gossip] header', id);
    });

    // store events

    gossip.on('replay', function (actions) {
      console.log('[gossip] list was reordered', row);
    });

    gossip.on('action', function (action, timestamp, source_id) {
      console.info('[gossip] recieved', action, timestamp, source_id);

      if (source_id !== gossip.id) {
        // this action was emitted elsewhere, so dispatch locally

        // try-catch to ensure update is applied correctly.
        // TODO: we might consider calling next() directly? is that idiomatic?
        try {
          store.dispatch(_extends({}, action, {
            __remote: true
          }));
        } catch (error) {
          console.error('ERROR DISPATCHING ACTION', error);
        }
      }
    });

    return function (next) {
      return function (action) {
        var nextState = next(action);

        history.actions.push(action);
        history.state.push(nextState);

        if (!action.__remote) {
          var result = gossip.localUpdate(action);

          // console.log('[gossip] action status', result)
          // console.log('[gossip] state', actions.asArray())
        }

        // return next(action)
        return nextState;
      };
    };
  };
};